/*
 Daybo Logic INI Support Library for non-Windows systems
 Copyright (C) 2001-2005 Daybo Logic.
 All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.
 3. Neither the name of the project nor the names of its contributors
    may be used to endorse or promote products derived from this software
    without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE PROJECT AND CONTRIBUTORS ``AS IS'' AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 ARE DISCLAIMED.  IN NO EVENT SHALL THE PROJECT OR CONTRIBUTORS BE LIABLE
 FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 SUCH DAMAGE.

  Daybo Logic,
  Flat 2 Barker Court,
  Victoria Avenue,
  Redfield,
  Bristol
  England.
  UK
  BS5 9NW

  +44 117 954 2306
  David Duncan Ross Palmer <http://www.daybologic.co.uk/mailddrp/>
  http://www.daybologic.co.uk/dev/dlini
*/

#ifndef INC_DLINI_H
#define INC_DLINI_H
/*-------------------------------------------------------------------------*/
/*
  Ensure all structure packing is aligned at DWORD boundries for compatibillity
  with VB and most other 32-bit applications.
*/
#if ( defined(DLINI_AS_DLL) && defined(__WIN32__) )
# include <pshpack4.h>
#endif

#ifdef __cplusplus
  extern "C" {
#endif /*__cplusplus*/

/* Private boolean type */
enum dlini_bool_t {
  dliniFalse = 0,
  dliniTrue = 1
};

/* Private error severity type */
enum dlini_errsev_t {
  dliniNotice,
  dliniWarning,
  dliniError
};

/* Version information */
#define DLINI_VER_MAJOR (0)
#define DLINI_VER_MINOR (2)
#define DLINI_VER_PATCH (1)

/*
  Visual C++ does this automatically for us so I might as well make use of it
*/
#if ( defined(DLINI_EXPORTS) || defined(DLINI_IMPORTS) )
#  ifndef DLINI_AS_DLL
#    define DLINI_AS_DLL
#  endif
#endif

#if ( defined(DLINI_AS_DLL) && defined(DLINI_SOURCE) && defined(__WIN32__) )
#  define DLINIAPI(ret) __declspec(dllexport) ##ret __stdcall
#elif ( defined(DLINI_AS_DLL) && !defined(DLINI_SOURCE) && defined(__WIN32__) )
#  define DLINIAPI(ret) __declspec(dllimport) ##ret __stdcall
#else
#  define DLINIAPI(ret) ret
#endif

#ifdef DLINI_SOURCE
  #define DLINI_LINEBUFFER (3001)

  #if ( !defined(__B_ENDIAN__) && !defined(__L_ENDIAN__) )
    #ifdef WORDS_BIGENDIAN /* GNU autoconf macro */
      #define __B_ENDIAN__
    #else
      #define __L_ENDIAN__
    #endif
  #endif

  /* List internal support function here, prefix with dlini_int_ */

  extern enum dlini_bool_t dlini_int_debugFlag;

  enum dlini_bool_t dlini_int_FindSection(
    const char *const FunctionName,
    FILE *FHandle,
    const char *SectionString /* Must be a resolved section name */
  );

  enum dlini_bool_t dlini_int_FindFirstSection(
    FILE *FHandle
  );

  enum dlini_bool_t dlini_int_FindNextSection(
    FILE *FHandle
  );

  void dlini_int_GoToLine(
    FILE *FHandle,
    unsigned int LineNumber
  );

  unsigned int dlini_int_GetKeyNames(
    FILE *FHandle,
    const char *SectionName,
    char *OutputBuffer,
    unsigned int BuffMaxChars
  );

  char *dlini_int_CreateResolvedSectionName(
    const char *SectionName
  );

  void dlini_int_DestroyResolvedSectionName(
    char *ResolvedSectionName
  );

  FILE *dlini_int_Open(
    const char *Filename,
    enum dlini_bool_t Write
  );

  void dlini_int_Close(
    FILE *FHandle
  );

  const char *dlini_int_ResolveFileName(
    const char *FileName
  );

  unsigned int dlini_int_ChopComment(
    char *LineString
  );

  unsigned int dlini_int_ChopCrap(
    char *LineString
  );

  enum dlini_bool_t dlini_int_FindKey(
    const char *const FunctionName,
    FILE *FHandle,
    const char *KeyName
  );

  enum dlini_bool_t dlini_int_FindNextKey(
    FILE *FHandle
  );

  unsigned int dlini_int_ReadKey(
    FILE *FHandle,
    char *lpReturnedString,
    unsigned int nSize,
    enum dlini_bool_t AttachEquals
  );

  enum dlini_bool_t dlini_int_FindNextValue(
    FILE *FHandle
  );

  unsigned int dlini_int_ReadValue(
    FILE *FHandle,
    char *lpReturnedString,
    unsigned int nSize
  );

  unsigned int dlini_int_StringToPositiveInteger(
    const char *NumberAsString
  );

  /*
    The following three functions are from DDRP's ANSI replacement set
    but I do not like interlibrary dependencies and they can become very
    messy quickly.  So they have been copied across with different names.
  */
  int dlini_int_stricmp(
    const char *const FunctionName,
    const char *s1,
    const char *s2
  );
  char *dlini_int_strlwr(
    char *s
  );
  char *dlini_int_strupr(
    char *s
  );

  /*
    This function is used to generate a checksum from a generic buffer.
    The checksum is one byte long only.  This is used with the .ini "struct"
    routines, which write a one byte checksum after the value data
  */
  unsigned char dlini_int_ByteChecksum(
    const void *Buffer,
    const unsigned int BufferSize
  );

  /*
    Case independent wrapper for strstr()
  */
  const char *dlini_int_stristr(
    const char *const FunctionName,
    const char *Str,
    const char *SubStr
  );

  /*
    strdup() (non-ANSI), replacement.  Will emit a warning on stderr if
    dlini_int_debugFlag is set and we fail to allocate a string large
    enough for the duplicate.
  */
  char *dlini_int_strdup(
    const char *const FunctionName,
    const char *Str
  );

#endif /*DLINI_SOURCE*/

DLINIAPI(int) dlini_Version(
  struct _S_DL_MODVER *PModVer
);

DLINIAPI(int) dlini_VersionEx(
  struct _S_DL_MODVER_EX *PModVer
);

/*
  To set the debug flag, send a pointer to an int which will set
  the flag.  The old status of the flag will be returned.  If NULL
  is sent, no changes will be made.
*/
DLINIAPI(int) dlini_DebugFlag(
  int *PBoolean
);

DLINIAPI(unsigned int) dlini_GetPrivateProfileInt(
    const char *lpAppName,      /* address of section name */
    const char *lpKeyName,      /* address of key name */
    int nDefault,                /* return value if key name is not found */
    const char *lpFileName /* address of initialization filename */
);

DLINIAPI(unsigned int) dlini_GetPrivateProfileSection(
    const char *lpAppName,  /* address of section name */
    char *lpReturnedString, /* address of return buffer */
    unsigned int nSize,     /* size of return buffer */
    const char *lpFileName  /* address of initialization filename */
);

DLINIAPI(unsigned int) dlini_GetPrivateProfileSectionNames(
    char *lpszReturnBuffer, /* address of return buffer */
    unsigned int nSize,     /* size of return buffer */
    const char *lpFileName       /* address of initialization filename */
);

DLINIAPI(unsigned int) dlini_GetPrivateProfileString(
    const char *lpAppName,                       /* points to section name */
    const char *lpKeyName,                           /* points to key name */
    const char *lpDefault,                     /* points to default string */
    char *lpReturnedString,                /* points to destination buffer */
    unsigned int nSize,                      /* size of destination buffer */
    const char *lpFileName            /* points to initialization filename */
);

DLINIAPI(int) dlini_GetPrivateProfileStruct( /* boolean return */
    const char *lpszSection,                    /* address of section name */
    const char *lpszKey,                            /* address of key name */
    void *lpStruct,                            /* address of return buffer */
    unsigned int uSizeStruct,                     /* size of return buffer */
    const char *szFile               /* address of initialization filename */
);

DLINIAPI(int) dlini_WritePrivateProfileSection( /* boolean return */
    const char *lpAppName,            /* pointer to string with section name */
    const char *lpString,             /* pointer to string with data */
    const char *lpFileName            /* pointer to string with filename */
);

DLINIAPI(int) dlini_WritePrivateProfileString( /* boolean return */
    const char *lpAppName,           /* pointer to section name */
    const char *lpKeyName,           /* pointer to key name */
    const char *lpString,            /* pointer to string to add */
    const char *lpFileName           /* pointer to initialization filename */
);

DLINIAPI(int) dlini_WritePrivateProfileStruct( /* boolean return */
    const char *lpszSection,         /* pointer to section name */
    const char *lpszKey,             /* pointer to key name */
    void *lpStruct,                  /* pointer to data to add */
    unsigned int uSizeStruct,        /* size, in bytes, of the buffer */
    const char *szFile               /* pointer to initialization filename */
);

/* In order to support Windows systems as well, I point back to the
Windows API when it's available. */
#if ( !defined(WIN32) && \
      !defined(__WIN32__) && \
      !defined(DLINI_SOURCE) || \
      defined(DLINI_FORCE_WIN32_INCLUSION) \
    )

# define GetPrivateProfileInt dlini_GetPrivateProfileInt
# define GetPrivateProfileSection dlini_GetPrivateProfileSection
# define GetPrivateProfileSectionNames dlini_GetPrivateProfileSectionNames
# define GetPrivateProfileString dlini_GetPrivateProfileString
# define GetPrivateProfileStruct dlini_GetPrivateProfileStruct
# define WritePrivateProfileSection dlini_WritePrivateProfileSection
# define WritePrivateProfileString dlini_WritePrivateProfileString
# define WritePrivateProfileStruct dlini_WritePrivateProfileStruct
#endif

/* Restore C++ naming */
#ifdef __cplusplus
  }
#endif

/*
  Restore alignment
*/
#if ( defined(DLINI_AS_DLL) && defined(__WIN32__) )
# include <poppack.h>
#endif
/*-------------------------------------------------------------------------*/
#endif /*!INC_DLINI_H*/
