/*
 Daybo Logic INI Support Library for non-Windows systems
 Copyright (C) 2001-2005 Daybo Logic.
 All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.
 3. Neither the name of the project nor the names of its contributors
    may be used to endorse or promote products derived from this software
    without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE PROJECT AND CONTRIBUTORS ``AS IS'' AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 ARE DISCLAIMED.  IN NO EVENT SHALL THE PROJECT OR CONTRIBUTORS BE LIABLE
 FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 SUCH DAMAGE.

  Daybo Logic,
  Flat 2 Barker Court,
  Victoria Avenue,
  Redfield,
  Bristol
  England.
  UK
  BS5 9NW

  +44 117 954 2306
  David Duncan Ross Palmer <http://www.daybologic.co.uk/mailddrp/>
  http://www.daybologic.co.uk/dev/dlini
*/

/*
   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
   % This module is a replacement for Windows'                 %
   % GetPrivateProfileInt() function.  It recovers an integer  %
   % from an ini file.  Read the Windows manual pages for info %
   % I try to adhere as closely as possible.                   %
   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
*/

#define DLINI_SOURCE
#include <limits.h>
#include <stdio.h>

#include "dlstdinc.h"
#include "dlini.h"
/*-------------------------------------------------------------------------*/
static const char *const FunctionName = "GetPrivateProfileInt";
/*-------------------------------------------------------------------------*/
DLINIAPI(unsigned int) dlini_GetPrivateProfileInt(
  const char *lpAppName,                        /* address of section name */
  const char *lpKeyName,                            /* address of key name */
  int nDefault,                   /* return value if key name is not found */
  const char *lpFileName             /* address of initialization filename */
)
{
  FILE* fileHandle;
  unsigned int ret = (unsigned int)nDefault;
  char* resolvedSectionName;
  char lpReturnedString[DLINI_LINEBUFFER];
  unsigned int nSize =
    (sizeof(lpReturnedString)/sizeof(lpReturnedString[0])) - 1;
  
  if ( !lpAppName || !lpKeyName ) /* The caller is in the wrong! */
    return ret;
    
  fileHandle = dlini_int_Open(
    dlini_int_ResolveFileName(lpFileName),
    dliniFalse
  );
  if ( !fileHandle ) return ret;
  
  resolvedSectionName = dlini_int_CreateResolvedSectionName(lpAppName);
  if ( resolvedSectionName ) {
    if (
      dlini_int_FindSection(FunctionName, fileHandle, resolvedSectionName)
    ) {
      if ( dlini_int_FindKey(FunctionName, fileHandle, lpKeyName) ) {
        if ( dlini_int_ReadValue(fileHandle, lpReturnedString, nSize) )
          ret = dlini_int_StringToPositiveInteger(lpReturnedString);
      }
    }
    dlini_int_DestroyResolvedSectionName(resolvedSectionName);
  }

  if ( fileHandle )
    dlini_int_Close(fileHandle);
  return ret;
}
/*-------------------------------------------------------------------------*/
