/*
 Daybo Logic INI Support Library for non-Windows systems
 Copyright (C) 2001-2005 Daybo Logic.
 All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.
 3. Neither the name of the project nor the names of its contributors
    may be used to endorse or promote products derived from this software
    without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE PROJECT AND CONTRIBUTORS ``AS IS'' AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 ARE DISCLAIMED.  IN NO EVENT SHALL THE PROJECT OR CONTRIBUTORS BE LIABLE
 FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 SUCH DAMAGE.

  Daybo Logic,
  Flat 2 Barker Court,
  Victoria Avenue,
  Redfield,
  Bristol
  England.
  UK
  BS5 9NW

  +44 117 954 2306
  David Duncan Ross Palmer <http://www.daybologic.co.uk/mailddrp/>
  http://www.daybologic.co.uk/dev/dlini
*/

/*
   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
   % This module is a replacement for Windows'                 %
   % GetPrivateProfileSectionNames() to return all names of    %
   % sections from an ini file.  Read the Windows manual pages %
   % for a definition, I try to adhere as closely as possible. %
   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
*/

#define DLINI_SOURCE
#include <stdio.h>
#include <limits.h>
#include <string.h>

#include "dlstdinc.h"
#include "dlini.h"
/*-------------------------------------------------------------------------*/
DLINIAPI(unsigned int) dlini_GetPrivateProfileSectionNames(
  char *lpszReturnBuffer,                      /* address of return buffer */
  unsigned int nSize,                             /* size of return buffer */
  const char *lpFileName             /* address of initialisation filename */
)
{
  FILE *fileHandle;
  char buff[DLINI_LINEBUFFER];
  unsigned int offset = 0U;

  if ( !lpszReturnBuffer ) {
    /*
      This is a required parameter.  I don't know exactly how we should
      handle this, as the manual isn't unambiguous.  So I am going to use
      the "buffer too small" way of doing things, returning nSize-2.
      However, if nSize is very small, I shall return zero.
    */
    if ( nSize >= 2 ) return nSize-2;
    return 0;
  }
  memset(lpszReturnBuffer, 0, nSize);

  fileHandle = dlini_int_Open(
    dlini_int_ResolveFileName(lpFileName),
    dliniFalse
  );
  if ( !fileHandle ) return 0; /* Can't open or lock file */

  rewind(fileHandle);
  while ( dlini_int_FindNextSection(fileHandle) ) {
    char *end;

    fgets(buff, sizeof(buff)/sizeof(buff[0]), fileHandle);
    end = strchr(buff, ']');
    if ( end ) {
      unsigned int charsCopied;

      *end = '\0';
      charsCopied = MIN(nSize-offset, strlen(buff)+1);

      memcpy(
        lpszReturnBuffer + offset,
        buff,
        charsCopied
      );
      offset += charsCopied;

      if ( strlen(buff) > nSize - offset ) {
        offset = nSize - 2;
        break;
      }
    }
  }

  dlini_int_Close(fileHandle);
  return offset;
}
/*-------------------------------------------------------------------------*/
