/*
 Daybo Logic INI Support Library for non-Windows systems
 Copyright (C) 2001-2005 Daybo Logic.
 All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.
 3. Neither the name of the project nor the names of its contributors
    may be used to endorse or promote products derived from this software
    without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE PROJECT AND CONTRIBUTORS ``AS IS'' AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 ARE DISCLAIMED.  IN NO EVENT SHALL THE PROJECT OR CONTRIBUTORS BE LIABLE
 FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 SUCH DAMAGE.

  Daybo Logic,
  Flat 2 Barker Court,
  Victoria Avenue,
  Redfield,
  Bristol
  England.
  UK
  BS5 9NW

  +44 117 954 2306
  David Duncan Ross Palmer <http://www.daybologic.co.uk/mailddrp/>
  http://www.daybologic.co.uk/dev/dlini
*/

/*
   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
   % This module is a replacement for Windows'                 %
   % GetPrivateProfileStruct() function.  This is an           %
   % interesting function which calculates a checksum, read    %
   % the Windows manual for more information.                  %
   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
*/

#define DLINI_SOURCE
#include <stdio.h>
#include <limits.h>
#include <string.h>

#include "dlstdinc.h"
#include "dlini.h"
/*-------------------------------------------------------------------------*/
static const char *const FunctionName = "GetPrivateProfileStruct";
/*-------------------------------------------------------------------------*/
DLINIAPI(int) dlini_GetPrivateProfileStruct(             /* boolean return */
  const char* lpszSection,                      /* address of section name */
  const char* lpszKey,                              /* address of key name */
  void* lpStruct,                              /* address of return buffer */
  unsigned int uSizeStruct,                       /* size of return buffer */
  const char* szFile                 /* address of initialization filename */
)
{
  FILE *fileHandle;
  char *resolvedSectionName;
  char buff[DLINI_LINEBUFFER];
  unsigned int buffSize = sizeof(buff)/sizeof(buff[0]);
  int ret = 0; /* Fail by default */

  memset(lpStruct, 0, uSizeStruct);

  fileHandle = dlini_int_Open(
    dlini_int_ResolveFileName(szFile),
    dliniFalse
  );
  if ( !fileHandle ) return ret; /* Can't open or lock file */

  resolvedSectionName = dlini_int_CreateResolvedSectionName(lpszSection);
  if ( resolvedSectionName ) {
    if (
      dlini_int_FindSection(FunctionName, fileHandle, resolvedSectionName)
    ) {
      if ( dlini_int_FindKey(FunctionName, fileHandle, lpszKey) ) {
        int valueLen = dlini_int_ReadValue(fileHandle, buff, buffSize);
        /* Now we have the encoded string, we need to crack it */
        if ( (valueLen % 2) != 0 ) {
          if ( dlini_int_debugFlag ) {
            fprintf(
              stderr,
              "%s(): Length of \"struct\" values must be even.\n",
              FunctionName
            );
          }
        }
        else {
          unsigned int i;
          for ( i = 0U; i < valueLen/2; i++ ) {
            unsigned char thisByte;
            int byteTransitional;
            sscanf(buff+(i*2), "%02X", &byteTransitional);
            thisByte = (unsigned char)byteTransitional;

            if ( i == (valueLen/2)-1 ) { /* Last, checksum byte */
              unsigned char actualSum =
                dlini_int_ByteChecksum(lpStruct, MIN(valueLen, uSizeStruct));
              if ( actualSum == thisByte ) {
                ret = 1; /* Success */
              }
              else {
                if ( dlini_int_debugFlag ) {
                  fprintf(
                    stderr,
                    "%s(): Checksum failed for [%s] %s in \"%s\"\n",
                    FunctionName,
                    lpszSection,
                    lpszKey,
                    szFile
                  );
                }
              }
            }
            else {
              if ( i <= uSizeStruct ) {
                memcpy(((unsigned char *)(lpStruct))+i, &thisByte, 1);
              }
            }
          }
        }
      }
    }
    dlini_int_DestroyResolvedSectionName(resolvedSectionName);
  }

  dlini_int_Close(fileHandle);
  return ret;
}
/*-------------------------------------------------------------------------*/
