/*
 Daybo Logic INI Support Library for non-Windows systems
 Copyright (C) 2001-2005 Daybo Logic.
 All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.
 3. Neither the name of the project nor the names of its contributors
    may be used to endorse or promote products derived from this software
    without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE PROJECT AND CONTRIBUTORS ``AS IS'' AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 ARE DISCLAIMED.  IN NO EVENT SHALL THE PROJECT OR CONTRIBUTORS BE LIABLE
 FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 SUCH DAMAGE.

  Daybo Logic,
  Flat 2 Barker Court,
  Victoria Avenue,
  Redfield,
  Bristol
  England.
  UK
  BS5 9NW

  +44 117 954 2306
  David Duncan Ross Palmer <http://www.daybologic.co.uk/mailddrp/>
  http://www.daybologic.co.uk/dev/dlini
*/

/*
   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
   % This module provides support to other INI functions only. %
   % The functions in here should not be used by users.        %
   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
*/

#define DLINI_SOURCE

#ifdef __UNIX__
# include <sys/file.h>
# include <unistd.h>
#else /* Win32 or DOS */
# if ( defined(__BORLANDC__) || defined(__WATCOMC__) )
#   include <io.h>
#   include <fcntl.h>
#   include <share.h>
#   include <sys/stat.h>
#    include <sys\types.h>
# else
#   error
# endif
#endif /*__UNIX__*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <stddef.h>
#include <limits.h>
#ifdef HDRSTOP
# pragma hdrstop
#endif /*HDRSTOP*/

#include "dlstdinc.h"
#include "dlini.h"
/*-------------------------------------------------------------------------*/
static enum dlini_bool_t SeekToKeysValue(
  FILE *FHandle
);

enum dlini_bool_t dlini_int_debugFlag = dliniFalse;
/*-------------------------------------------------------------------------*/
enum dlini_bool_t dlini_int_FindSection(
  const char *const FunctionName,
  FILE *FHandle,
  const char *SectionString
)
{
  char lineBuff[DLINI_LINEBUFFER];
  char* s;
  
  rewind(FHandle);
  do {
    s = fgets(lineBuff, sizeof(lineBuff)/sizeof(lineBuff[0]), FHandle);
    if ( s ) {
      dlini_int_ChopComment(s);
      dlini_int_ChopCrap(s);
      if ( dlini_int_stricmp(FunctionName, s, SectionString) == 0 )
        return dliniTrue;
    }
  } while ( s );
  rewind(FHandle);
  return dliniFalse;
}
/*-------------------------------------------------------------------------*/
enum dlini_bool_t dlini_int_FindFirstSection(
  FILE *FHandle
)
{
  if ( FHandle ) rewind(FHandle);
  return dlini_int_FindNextSection(FHandle);
}
/*-------------------------------------------------------------------------*/
enum dlini_bool_t dlini_int_FindNextSection(
  FILE *FHandle
)
{
  enum dlini_bool_t lastCharEOL = dliniFalse;
  int currentChar;

  if ( !FHandle ) return dliniFalse;

  do {
    currentChar = fgetc(FHandle);
    if ( lastCharEOL && currentChar == '[' )
      return dliniTrue;

    if ( currentChar == '\n' )
      lastCharEOL = dliniTrue;
  } while ( currentChar != EOF );
  return dliniFalse;
}
/*-------------------------------------------------------------------------*/
void dlini_int_GoToLine(
  FILE *FHandle,
  unsigned int LineNumber
)
{
  rewind(FHandle);
  while ( LineNumber ) {
    int byte = fgetc(FHandle);
    if ( byte == EOF ) break;
    if ( byte == '\n' ) LineNumber--;
  }
}
/*-------------------------------------------------------------------------*/
unsigned int dlini_int_GetKeyNames(
  FILE *FHandle,
  const char *SectionName,
  char *OutputBuffer,
  unsigned int BuffMaxChars
)
{
}
/*-------------------------------------------------------------------------*/
char *dlini_int_CreateResolvedSectionName(
  const char *SectionName
)
{
  char *ptr;
  size_t sz;

  sz = ( strlen(SectionName)+3 ) * sizeof(SectionName[0]);
  ptr = (char*)malloc( sz );
  if ( ptr ) {
    ptr[0] = '[';
    strcpy(ptr+1, SectionName);
    strcat(ptr, "]");
  }

  return ptr;
}
/*-------------------------------------------------------------------------*/
void dlini_int_DestroyResolvedSectionName(
  char *ResolvedSectionName
)
{
  if ( ResolvedSectionName )
    free(ResolvedSectionName);
}
/*-------------------------------------------------------------------------*/
FILE *dlini_int_Open(
  const char *Filename,
  enum dlini_bool_t Write
)
{
  /* This function is where I have to deal with locks.
  This makes this code pretty much a hack and not portable.
  I shall add new code for different ways of doing things as things
  break. */

  int fd;
  FILE *fsp = NULL;

# ifdef __UNIX__
    struct flock fl;

    fl.l_type = ( (Write==dliniTrue) ? (F_WRLCK) : (F_RDLCK) );
    fl.l_whence = SEEK_SET; /* Values below absolute */
    fl.l_start = 0; /* Lock section starts at top */
    fl.l_len = 0; /* Indicates entire file */
    fl.l_pid = getpid(); /* Identify process which owns the lock (us) */

    fd = open(Filename, ( (Write==dliniTrue) ? (O_RDWR) : (O_RDONLY) ));
    if ( fd != -1 ) { /* If no error */
      if ( fcntl(fd, F_SETLKW, &fl) == -1 ) {
        close(fd);
        fd = -1;
      }
    }
    else { /* -1 error on open */
      perror("INI open failed:");
      return NULL;
    }
# else /*__WIN32__*/
#   if ( defined(__BORLANDC__) || defined(__WATCOMC__) )
      fd = sopen(Filename, O_RDONLY, SH_DENYRW, S_IREAD); /* Fix this */
      if ( fd != -1 ) {
        if ( lock(fd, 0L, filelength(fd)) ) {
          /* Locking the file has failed */
          close(fd);
          fd = -1;
        }
      }
#   else
#     error ("No code for this compiler")
#   endif /*__BORLANDC__*/
# endif /*__UNIX__*/

  if ( fd != -1 ) { /* If no error */
    fsp = fdopen(fd, ( (Write==dliniTrue) ? ("r+t") : ("rt") ));
    if ( !fsp )
      close(fd);
  }

  return fsp;
}
/*-------------------------------------------------------------------------*/
void dlini_int_Close(
  FILE *FHandle
)
{
  if ( FHandle ) {
    /* I don't think I have to unlock the file in UNIX, just close the handle
    and I should be finished with it. */
#ifndef __UNIX__ /* Not UNIX */
# if ( defined(__BORLANDC__) || defined(__WATCOMC__) )
    int fd = fileno(FHandle);
    unlock(fd, 0L, filelength(fd));
# else
#   error "Can't handle compiler"
# endif
#endif
    fclose(FHandle);
  }
}
/*-------------------------------------------------------------------------*/
const char *dlini_int_ResolveFileName(
  const char *FileName
)
{
  /* This is what Windows calls the default INI file anyroad */
  static const char defFileName[] = "win.ini";

  if ( FileName ) return FileName;
  return defFileName;
}
/*-------------------------------------------------------------------------*/
unsigned int dlini_int_ChopComment(
  char *LineString
)
{
  unsigned int oldLen;
  char *comment;

  oldLen = strlen(LineString);
  comment = strchr(LineString, ';');
  if ( comment )
    comment[0] = '\0'; /* Terminate at beginning of comment */
    
  return oldLen - strlen(LineString);
}
/*-------------------------------------------------------------------------*/
unsigned int dlini_int_ChopCrap(
  char *LineString
)
{
  size_t pos;
  unsigned int oldLen;

  pos = strlen(LineString);
  oldLen = pos;
  if ( pos-- ) {
    do {
      if (
        LineString[pos] == '\n' ||
        LineString[pos] == ' ' ||
        LineString[pos] == '\r'
      ) {
        LineString[pos] = '\0';
        break;
      }
    } while ( pos-- );
  }
  return oldLen - strlen(LineString);
}
/*-------------------------------------------------------------------------*/
enum dlini_bool_t dlini_int_FindKey(
  const char *const FunctionName,
  FILE *FHandle,
  const char *KeyName
)
{
  char lineBuff[DLINI_LINEBUFFER];
  char* s;
  
  do {
    const char *keyoff; /* Offset of key into string (must be zero) */
    s = fgets(lineBuff, sizeof(lineBuff)/sizeof(lineBuff[0]), FHandle);
    if ( !s ) break; /* EOF */

    if ( lineBuff[0] == '[' ) /* Started new section */
      return dliniFalse; /* Do not look in the next section */
      
    keyoff = dlini_int_stristr(FunctionName, s, KeyName);
    if ( keyoff ) {
      if ( keyoff == lineBuff ) /* Key at beginning of line; perfect */
        return SeekToKeysValue(FHandle);
    }
  } while ( s );
  
  return dliniFalse;
}
/*-------------------------------------------------------------------------*/
enum dlini_bool_t dlini_int_FindNextKey(
  FILE *FHandle
)
{
  if ( FHandle ) {
    enum dlini_bool_t loop = dliniTrue;
    char lastChar = '\n';
    int currentChar;

    do {
      currentChar = fgetc(FHandle);
      if ( currentChar == EOF )
        return dliniFalse;
        
      if ( (char)lastChar == '\n' ) {
        if (
          currentChar != ' ' &&
          currentChar != '\n' &&
          currentChar != '['
        ) {
          fseek(FHandle, -1, SEEK_CUR); /* Back one char */
          break;
        }
      }

      lastChar = (char)currentChar;
    } while ( loop );
  }
  else
    return dliniFalse; /* File handle not supplied */

  return dliniTrue;
}
/*-------------------------------------------------------------------------*/
unsigned int dlini_int_ReadKey(
  FILE *FHandle,
  char *lpReturnedString,
  unsigned int nSize,
  enum dlini_bool_t AttachEquals
)
{
  int currentChar;
  char buff[DLINI_LINEBUFFER];
  unsigned int readChars = 0U;

  memset(lpReturnedString, 0, nSize);
  do {
    currentChar = fgetc(FHandle);
    if ( (char)currentChar == ';' ) { /* Start of comment? */
      fgets(buff, sizeof(buff), FHandle); /* Read (discard) rest of line */
      continue; /* Start over, junking comment */
    }
    else if ( (char)currentChar == '=' ) { /* separator encountered */
      fseek(FHandle, -1, SEEK_CUR); /* Rewind one character */
      break;
    }
    else {
      buff[readChars] = (char)currentChar;
      readChars++;
    }
  } while ( !feof(FHandle) );

  buff[readChars] = '\0';
  readChars -= dlini_int_ChopCrap(buff);
  if ( AttachEquals ) {
    if ( readChars < (sizeof(buff)/sizeof(buff[0])-1) ) {
      buff[readChars] = '=';
      buff[readChars+1] = '\0';
    }
  }
  strncat(lpReturnedString, buff, nSize);
  return readChars;
}
/*-------------------------------------------------------------------------*/
enum dlini_bool_t dlini_int_FindNextValue(
  FILE *FHandle
)
{
  if ( FHandle ) {
    char currentChar;
    enum dlini_bool_t loop = dliniTrue;
    enum dlini_bool_t equalsFound = dliniFalse;

    do {
      currentChar = fgetc(FHandle);
      if ( currentChar == '=' )
        equalsFound = dliniTrue;
      else if ( currentChar != ' ' ) {
        loop = false;
        fseek(FHandle, -1, SEEK_CUR);
      }

    } while ( loop );

    if ( equalsFound ) return dliniTrue;
  }
  
  return dliniFalse;
}
/*-------------------------------------------------------------------------*/
unsigned int dlini_int_ReadValue(
  FILE *FHandle,
  char *lpReturnedString,
  unsigned int nSize
)
{
  memset(lpReturnedString, 0, nSize);
  fgets(lpReturnedString, nSize, FHandle);
  dlini_int_ChopComment(lpReturnedString);
  dlini_int_ChopCrap(lpReturnedString);
  return strlen(lpReturnedString);
}
/*-------------------------------------------------------------------------*/
int dlini_int_stricmp(
  const char *const FunctionName,
  const char *s1,
  const char *s2
)
{
  char *copyS1;
  char *copyS2; /* Copies of both */
  int ret; /* Return value */
  
  copyS1 = dlini_int_strdup(FunctionName, s1);
  copyS2 = dlini_int_strdup(FunctionName, s2);
  if ( !copyS1 || !copyS2 ) {
    if ( dlini_int_debugFlag ) {
      fprintf(
        stderr,
        "%s(): Not enough memory to make a comparison in "
        "dlini_int_stricmp().  Comparison will be negative\n",
        FunctionName
      );
    }
    return -1;
  }

  /* Convert both to uppercase which provides a way to ignore the case */  
  dlini_int_strupr(copyS1);
  dlini_int_strupr(copyS2);
  
  ret = strcmp(copyS1, copyS2); /* Do comparison and save result */
  
  /* Cleanup and return */
  free(copyS1); free(copyS2);
  return ret;
}
/*-------------------------------------------------------------------------*/
char *dlini_int_strlwr(
  char *s
)
{
  unsigned int index = 0U;
  
  if ( !s ) return s; /* Get out before damage is done */
  while ( s[index] != '\0' ) {
    s[index] = (char)tolower( (int)(s[index]) );
    index++;
  };
  return s;
}
/*-------------------------------------------------------------------------*/
char *dlini_int_strupr(
  char *s
)
{
  unsigned int index = 0U;
  
  if ( !s ) return s; /* Get out before damage is done */
  while ( s[index] != '\0' ) {
    s[index] = (char)toupper( (int)(s[index]) );
    index++;
  };
  return s;
}
/*-------------------------------------------------------------------------*/
const char *dlini_int_stristr(
  const char *const FunctionName,
  const char *Str,
  const char *SubStr
)
{
  char *strCopy;
  char *subStrCopy;
  const char *ret = NULL;

  strCopy = dlini_int_strdup(FunctionName, Str);
  subStrCopy = dlini_int_strdup(FunctionName, SubStr);
  if ( strCopy && subStrCopy ) {
    char *offset;
    dlini_int_strlwr(strCopy);
    dlini_int_strlwr(subStrCopy);

    offset = strstr(strCopy, subStrCopy);
    if ( offset ) {
      unsigned int diff = offset - strCopy;
      ret = Str + diff;
    }
    
    free(strCopy);
    free(subStrCopy);
  }

  return ret;
}
/*-------------------------------------------------------------------------*/
char *dlini_int_strdup(
  const char *const FunctionName,
  const char *Str
)
{
  char *strCpy = NULL;
  const char *const thisFunctionName = "dlini_int_strdup";

  if ( Str ) {
    unsigned int bytes = strlen(Str)+1;
    strCpy = malloc(bytes);
    if ( strCpy ) {
      strcpy(strCpy, Str);
    }
    else {
      fprintf(
        stderr,
        "%s(): Failed to allocate %u bytes with %s()\n\n",
        (FunctionName) ? (FunctionName) : ("unknown"),
        bytes,
        thisFunctionName
      );
    }
  }
  else {
    fprintf(
      stderr,
      "%s(): Sent a NULL string to %s()\n",
      (FunctionName) ? (FunctionName) : ("unknown"),
      thisFunctionName
    );
  }

  return strCpy;
}
/*-------------------------------------------------------------------------*/
static enum dlini_bool_t SeekToKeysValue(
  FILE *FHandle
)
{
  unsigned int i = 2;
  int c;
  
  while ( i ) {
    long int oldPos = ftell(FHandle);
    
    fseek(FHandle, -1, SEEK_CUR); /* Go back one byte */
    
    if ( ftell(FHandle) != (oldPos - 1) )
      return dliniFalse;
      
    c = getc(FHandle);
    if ( c == '\n' )
      i--;
      
    ungetc(c, FHandle);
  }
  /*
    We are now at the start of the line we want.  Seek forward until we
    discover the '=', when we do, we're in front of it ready for whoever
    wants to read or write the value
  */
  
  do {
    c = fgetc(FHandle);
    if ( c == '=' ) break;
    if ( c == EOF ) return dliniFalse;
  } while ( c != EOF ); /* Condiction is of no consequence */
  
  return dliniTrue;
}
/*-------------------------------------------------------------------------*/
unsigned int dlini_int_StringToPositiveInteger(
  const char *StringAsNumber
)
{
  int number = atoi(StringAsNumber);
  if ( number < 0 ) number = 0; /* Microsoft rules, negatives are illegal */
  return number;
}
/*-------------------------------------------------------------------------*/
unsigned char dlini_int_ByteChecksum(
  const void *Buffer,
  const unsigned int BufferSize
)
{
  unsigned char sum = '\0';
  unsigned int i;
  unsigned char *CharBuffer = (unsigned char *)Buffer;

  for ( i = 0U; i < BufferSize; i++ )
    sum += CharBuffer[i];

  return sum;
}
/*-------------------------------------------------------------------------*/
