/*
 Daybo Logic INI Library
 Copyright (C) 2001-2005 Daybo Logic.
 All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.
 3. Neither the name of the project nor the names of its contributors
    may be used to endorse or promote products derived from this software
    without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE PROJECT AND CONTRIBUTORS ``AS IS'' AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 ARE DISCLAIMED.  IN NO EVENT SHALL THE PROJECT OR CONTRIBUTORS BE LIABLE
 FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 SUCH DAMAGE.

  Daybo Logic,
  Flat 2 Barker Court,
  Victoria Avenue,
  Redfield,
  Bristol
  England.
  UK
  BS5 9NW

  +44 117 954 2306
  David Duncan Ross Palmer <http://www.daybologic.co.uk/mailddrp/>
  http://www.daybologic.co.uk/dev/dlini
*/

/*
   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
   % This program is meant for debugging and developing dlini  %
   % It has no useful purpose and is not built in with the lib %
   % file.                                                     %
   %     I have defined DLINI_FORCE_WIN32_INCLUSION (handled   %
   % in dlini.h) to ensure the Win32 functions are not used.   %
   % If I did not define this, dlini.h would assume because we %
   % might be building on Windows, on that platform, we should %
   % defer to those functions, to reduce binary size.          %
   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
*/

#define DLINI_FORCE_WIN32_INCLUSION
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <limits.h>
#ifdef HDRSTOP
# pragma hdrstop
#endif /*HDRSTOP*/

#include "dlstdinc.h"
#include "dlini.h"
/*-------------------------------------------------------------------------*/
#define TEST1(name) \
          static enum dlini_errsev_t name(void)
#define TEST2(name) \
          static enum dlini_errsev_t name()

#define INIFILE "./sample.ini"

TEST1(Version);
TEST1(ReadHouses);
TEST1(WriteHouses);
TEST1(ReadFishInTheSea);
TEST1(WriteFishInTheSea);
TEST1(NoSpaceSection);
TEST1(SpacedString);
TEST1(ReadSectionNames);
TEST1(ReadSection);
TEST1(ReadStruct);

static const char *th(unsigned int Number);
static void Title(void);
/*-------------------------------------------------------------------------*/
int main()
{
  unsigned int i;
  int result = EXIT_SUCCESS;

  enum dlini_errsev_t (*Tests[])(void) = {
    Version,
    ReadHouses,
    ReadFishInTheSea,
    NoSpaceSection,
    SpacedString,
    ReadSectionNames,
    ReadSection,
    ReadStruct,
    WriteFishInTheSea,
    WriteHouses,
  };

  const char *const TestNames[] = {
    "Version",
    "ReadHouses",
    "ReadFishInTheSea",
    "NoSpaceSection",
    "SpacedString",
    "ReadSectionNames",
    "ReadSection",
    "ReadStruct",
    "WriteFishInTheSea",
    "WriteHouses",
  };

  Title();
  for ( i = 0U; i < sizeof(Tests)/sizeof(Tests[0]); i++ ) {
    if ( Tests[i] ) {
      enum dlini_errsev_t severity = Tests[i]();
      if ( severity == dliniNotice ) {
        printf(
          "%u%s test (%s) succeeded\n",
          i+1,
          th(i+1),
          TestNames[i]
        );
      }
      else if ( severity == dliniWarning ) {
        printf(
          "%u%s test (%s) (*FAILED*).  -- Ignoring\n",
          i+1,
          th(i+1),
          TestNames[i]
        );
      }
      else {
        printf(
          "%u%s test (%s) (*FAILED*).\n",
          i+1,
          th(i+1),
          TestNames[i]
        );
        result = EXIT_FAILURE;
        break;
      }
    }
    else
      puts("Warning Missing handler in tests loop");
  }
  return result;
}
/*-------------------------------------------------------------------------*/
TEST2(Version)
{
  S_DL_MODVER modVer;

  modVer.size = sizeof(modVer);
  if ( dlini_Version(&modVer) ) {
    if ( modVer.major != DLINI_VER_MAJOR ) return dliniError;
    if ( modVer.minor != DLINI_VER_MINOR ) return dliniError;
    if ( modVer.build != DLINI_VER_PATCH ) return dliniError;
    return dliniNotice;
  }

  return dliniError;
}
/*-------------------------------------------------------------------------*/
TEST2(ReadHouses)
{
  const char houses[] = "houses";
  const char eight[] = "eight";
  char buff[256];
  unsigned int n; /* Number of characters read */

  n = GetPrivateProfileString(
    "last_section",
    houses,
    "none",
    buff,
    (sizeof(buff)/sizeof(buff[0])),
    INIFILE
  );

  if ( n == strlen(eight) ) {
    if ( strcmp(buff, eight) == 0 )
      return dliniNotice;
  }

  printf(
    "ReadHouses fails because we expect \"%s\" %s and actually get "
   "\"%s\"\n",
   eight,
   houses,
   buff
  );
  return dliniError;
}
/*-------------------------------------------------------------------------*/
TEST2(WriteHouses)
{
  return dliniWarning; /*FIXME*/
}
/*-------------------------------------------------------------------------*/
TEST2(ReadFishInTheSea)
{
  const char key[] = "fish_in_the_sea";
  const char section[] = "section_two";
  const unsigned int expected = 55809;
  unsigned int real;

  real = GetPrivateProfileInt(section, key, 0, INIFILE);

  if ( !real ) {
    puts("GetPrivateProfileInt() returned zero!");
    return dliniError;
  }
  if ( real != expected ) {
    puts("GetPrivateProfileInt() returned an unexpected nonzero");
    return dliniError;
  }

  return dliniNotice; /* Yeah! */
}
/*-------------------------------------------------------------------------*/
TEST2(WriteFishInTheSea)
{
  return dliniWarning; /*FIXME*/
}
/*-------------------------------------------------------------------------*/
TEST2(NoSpaceSection)
{
  const char section[] = "no_space_section";
  const char key[] = "value";
  const char valueDefault[] = "nonsense";
  const char expected[] = "overlord";
  char buff[256];
  size_t nSize = sizeof(buff)/sizeof(buff[0]);
  unsigned int n;

  n = GetPrivateProfileString(
    section,
    key,
    valueDefault,
    buff,
    nSize,
    INIFILE
  );

  if ( n == strlen(expected) ) {
    if ( strcmp(buff, expected) == 0 )
      return dliniNotice;
  }

  return dliniError;
}
/*-------------------------------------------------------------------------*/
TEST2(SpacedString)
{
  const char section[] = "no_space_section";
  const char key[] = "spaced_string";
  const char valueDefault[] = "bugger";
  const char expected[] = "what you say";
  char buff[256];
  size_t nSize = ( sizeof(buff)/sizeof(buff[0]) );
  unsigned int n;

  n = GetPrivateProfileString(
    section,
    key,
    valueDefault,
    buff,
    nSize,
    INIFILE
  );

  if ( n == strlen(expected) ) {
    if ( strcmp(buff, expected) == 0 )
      return dliniNotice;
  }

  return dliniError;
}
/*-------------------------------------------------------------------------*/
TEST2(ReadSectionNames)
{
  const char *expectSecNames[] = {
    "section_one",
    "section_two",
    "no_space_section",
    "last_section",
    "\0" /* This makes sure the lib returns two NULLs at the end */
  };
  char buff[256];
  size_t nSize = sizeof(buff)/sizeof(buff[0]);
  unsigned int n, expectIndex;
  unsigned int expectCount =
    sizeof(expectSecNames)/sizeof(expectSecNames[0]);
  char *actualPtr = buff;

  n = GetPrivateProfileSectionNames(
    buff,
    nSize,
    INIFILE
  );

  if ( n == nSize-2 ) {
    printf(
      "ERROR: buff[%u] not large enough to contain section names",
      (unsigned int)sizeof(buff),
      stderr
    );
    return dliniError;
  }

  if ( !n ) {
    fputs(
      "Zero return from GetPrivateProfileSectionNames()\n",
      stderr
    );
    return dliniError;
  }

  if ( n != 54 ) {
    fputs(
      "ERROR: The test expects the return value of "
      "GetPrivateProfileSectionNames() to be 54\n",
      stderr
    );
    return dliniError;
  }

  for ( expectIndex = 0U; expectIndex < expectCount; expectIndex++ ) {
    if ( strcmp(actualPtr, expectSecNames[expectIndex]) != 0 ) {
      fprintf(stderr, "***ERROR: Mismatch in ReadSectionNames()\n");
      return dliniError;
    }
    actualPtr += strlen(expectSecNames[expectIndex]) + 1;
  }

  return dliniNotice;
}
/*-------------------------------------------------------------------------*/
TEST2(ReadSection)
{
  const char *expectedSection[] = {
    "beans=5",
    "nothing=",
    "houses=eight",
    "equals=spaced",
    "name=david duncan ross palmer",
    "name_with_correct_case=David Duncan Ross Palmer",
    "\0" /* This makes sure the lib returns two NULLs at the end */
  };
  char buff[4096];
  char *actualPtr = buff;
  unsigned int nSize = sizeof(buff)/sizeof(buff[0]);
  unsigned int n, expectIndex;
  unsigned int expectCount =
    sizeof(expectedSection)/sizeof(expectedSection[0]);

  n = GetPrivateProfileSection(
    "last_section",
    buff,
    nSize,
    INIFILE
  );

  if ( n == nSize-2 ) {
    fprintf(
      stderr,
      "buff[%u]: The buffer is too small to hold the result\n",
      nSize
    );
    return dliniError;
  }

  if ( !n ) {
    fputs(
      "Zero return from GetPrivateProfileSection()\n",
      stderr
    );
    return dliniError;
  }

  if ( n != 122 ) {
    fprintf(
      stderr,
      "ERROR: GetPrivateProfile() returned %u, expected 122\n",
      n
    );
    return dliniError;
  }

  for ( expectIndex = 0U; expectIndex < expectCount; expectIndex++ ) {
    if ( strcmp(actualPtr, expectedSection[expectIndex]) != 0 ) {
      fprintf(
        stderr,
        "Mismatch in ReadSection(), expected \"%s\" and got \"%s\"\n",
        expectedSection[expectIndex],
        actualPtr
      );
      /*return dliniFalse;*/ /* FIXME */
    }
    actualPtr += strlen(expectedSection[expectIndex]) + 1;
  }

  return dliniNotice;
}
/*-------------------------------------------------------------------------*/
TEST2(ReadStruct)
{
  int result;
  const char expectDavid[] = "David";
  const char expectRegistry[] = "Registry users are muppets";
  char actualDavid[sizeof(expectDavid)/sizeof(expectDavid[0])];
  char actualRegistry[sizeof(expectRegistry)/sizeof(expectRegistry[0])];

  memset(actualDavid, 0, sizeof(actualDavid));
  memset(actualRegistry, 0, sizeof(actualRegistry));

  result = GetPrivateProfileStruct(
    "section_one",
    "David",
    (void *)actualDavid,
    strlen(expectDavid),
    INIFILE
  );

  if ( result ) {
    if ( memcmp(expectDavid, actualDavid, strlen(expectDavid)) ) {
      fprintf(
        stderr,
        "Mismatch in sub-test 1: \"%s\" is not \"%s\"\n",
        actualDavid,
        expectDavid
      );
      return dliniError;
    }
  }

  result = GetPrivateProfileStruct(
    "SECTION_TWO",
    "REGISTRY",
    (void *)actualRegistry,
    strlen(expectRegistry),
    INIFILE
  );

  if ( result ) {
    if ( memcmp(expectRegistry, actualRegistry, strlen(expectRegistry)) ) {
      fprintf(
        stderr,
        "Mismatch in sub-test 2: \"%s\" is not \"%s\"\n",
        actualRegistry,
        expectRegistry
      );
      return dliniError;
    }
    else {
      return dliniNotice;
    }
  }

  return dliniError;
}
/*-------------------------------------------------------------------------*/
static const char *th(unsigned int Number)
{
  const char *st = "st";
  const char *nd = "nd";
  const char *rd = "rd";
  const char *th = "th";

  if ( Number >= 4 && Number <= 20 )
    return th;

  if ( Number % 4 == 3 )
    return rd;

  if ( Number % 3 == 2 )
    return nd;

  if ( Number % 2 == 1 )
    return st;

  return th;
}
/*-------------------------------------------------------------------------*/
static void Title()
{
  puts("libdlini is supplied under a BSD-style license");
  puts("This program will test it's functionallity");
  puts("(C)2001-2005 Daybo Logic.  http://www.daybologic.co.uk/mailddrp");
  puts("WRITING FUNCTIONS WILL FAIL: DO NOT BE ALARMED, THEY DO NOT WORK\n");
}
/*-------------------------------------------------------------------------*/
